function [x,fval] = optimNested(a,b,c,d,e)
%[X,FVAL] = optimNested(A,B,C,D,E) solves an unconstrained
%   optimization problem in two dimensions using nested functions
%
%   Example
%  % 1) Smooth objective function
%     [x, fval ] = optimNested(3,2,1,0,0)
%
%  % 2) Non-smooth objective function
%     [x, fval ] = optimNested(3,2,1,5,0)
%
%  % 3) Stochastic objective function
%     [x, fval ] = optimNested(3,2,1,0,2)
%
%  % 4) Non-smooth stochastic objective function
%     [x, fval ] = optimNested(3,2,1,6,3)

    if nargin < 5
        %Defaults
        a = 3; b = 2; c = 1; d = 6; e = 0;
    end
    
    %Settings
    range = [-10 10; -10 10];
    x0 = [-5,9];     % Starting guess
    options = optimoptions('fminunc','algorithm','quasi-newton',...
                  'OutputFcn',@myoutputfcn,'Display','off');
    
    %Minimize!          
    [x, fval] = fminunc(@objfun,x0,options);

    % Nested Functions
    function f = objfun(x)
        % Nonlinear objective function
        f = a*x(:,1).^2 + b*x(:,1).*x(:,2) + ...
            c*x(:,2).^2 + d*abs(x(:,2) - x(:,1))+ e*randn;
    end

    function stop = myoutputfcn(x,optimvalues,state)
        stop = false;
        switch state
            case 'init'
                clf
                plotobjective(@(x) objfun(x),range);
            case 'iter'
                plot3(x(1),x(2),optimvalues.fval+2, ...
                    'ro','MarkerFaceColor','r', ...
                    'MarkerSize',6)
                pause(0.5);
            case 'done'
                plot3(x(1),x(2),optimvalues.fval+5, ...
                    'y*','MarkerFaceColor','y', ...
                    'MarkerSize',10)

        end
    end
end

